! (C) Copyright 2014- ECMWF.
!
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
!
! In applying this licence, ECMWF does not waive the privileges and immunities
! granted to it by virtue of its status as an intergovernmental organisation
! nor does it submit to any jurisdiction.

SUBROUTINE COML_SET_DEBUG(KONOFF, KRET)
USE OML_MOD, ONLY : OML_DEBUG
USE PARKIND1, ONLY : JPIM
IMPLICIT NONE
INTEGER(KIND=JPIM), INTENT(IN) :: KONOFF
INTEGER(KIND=JPIM), INTENT(OUT) :: KRET
KRET = 0
IF (OML_DEBUG) KRET = 1
IF (KONOFF == 0) THEN 
  OML_DEBUG = .FALSE.
ELSE
  OML_DEBUG = .TRUE.
ENDIF
END SUBROUTINE COML_SET_DEBUG

SUBROUTINE COML_INIT_LOCKID_WITH_NAME(KMYLOCK,CDLOCKNAME)
USE OML_MOD, ONLY : OML_INIT_LOCK, OML_LOCK_KIND, OML_DEBUG
USE PARKIND1, ONLY : JPIB
IMPLICIT NONE
INTEGER(KIND=OML_LOCK_KIND), INTENT(INOUT) :: KMYLOCK
CHARACTER(LEN=*), INTENT(IN) :: CDLOCKNAME
INTEGER(KIND=JPIB), EXTERNAL :: LOC_ADDR
CALL OML_INIT_LOCK(KMYLOCK)
IF (OML_DEBUG) WRITE(0,'(1x,a,2i20)') &
     & 'coml_init_lockid_with_name "'//CDLOCKNAME//'" :',KMYLOCK,LOC_ADDR(KMYLOCK)
END SUBROUTINE COML_INIT_LOCKID_WITH_NAME

SUBROUTINE COML_INIT_LOCKID(KMYLOCK)
USE OML_MOD, ONLY : OML_INIT_LOCK, OML_LOCK_KIND, OML_DEBUG
USE PARKIND1, ONLY : JPIB
IMPLICIT NONE
INTEGER(KIND=OML_LOCK_KIND), INTENT(INOUT) :: KMYLOCK
INTEGER(KIND=JPIB), EXTERNAL :: LOC_ADDR
CALL OML_INIT_LOCK(KMYLOCK)
IF (OML_DEBUG) WRITE(0,'(1x,2i20)') &
     & 'coml_init_lockid :',KMYLOCK,LOC_ADDR(KMYLOCK)
END SUBROUTINE COML_INIT_LOCKID

SUBROUTINE COML_INIT_LOCK()
USE OML_MOD, ONLY : OML_INIT_LOCK
IMPLICIT NONE
CALL OML_INIT_LOCK()
END SUBROUTINE COML_INIT_LOCK

SUBROUTINE COML_TEST_LOCKID(KISSET,KMYLOCK)
USE PARKIND1, ONLY : JPIM
USE OML_MOD, ONLY : OML_TEST_LOCK, OML_LOCK_KIND
IMPLICIT NONE
INTEGER(KIND=JPIM), INTENT(OUT) :: KISSET
INTEGER(KIND=OML_LOCK_KIND), INTENT(INOUT) :: KMYLOCK
KISSET = 1
IF (.NOT.OML_TEST_LOCK(KMYLOCK)) KISSET = 0
END SUBROUTINE COML_TEST_LOCKID

SUBROUTINE COML_TEST_LOCK(KISSET)
USE PARKIND1, ONLY : JPIM
USE OML_MOD, ONLY : OML_TEST_LOCK
IMPLICIT NONE
INTEGER(KIND=JPIM), INTENT(OUT) :: KISSET
KISSET = 1
IF (.NOT.OML_TEST_LOCK()) KISSET = 0
END SUBROUTINE COML_TEST_LOCK

SUBROUTINE COML_SET_LOCKID(KMYLOCK)
USE OML_MOD, ONLY : OML_SET_LOCK, OML_LOCK_KIND, OML_MY_THREAD, OML_DEBUG
USE PARKIND1, ONLY : JPRD, JPIB
IMPLICIT NONE
INTEGER(KIND=OML_LOCK_KIND), INTENT(INOUT) :: KMYLOCK
INTEGER(KIND=JPIB), EXTERNAL :: LOC_ADDR
REAL(KIND=JPRD), EXTERNAL :: UTIL_WALLTIME
IF (OML_DEBUG) WRITE(0,'(1x,f20.6,1x,i3,a,2i20)') &
     & UTIL_WALLTIME(),OML_MY_THREAD(),': coml_SET_lockid >>',KMYLOCK,LOC_ADDR(KMYLOCK)
CALL OML_SET_LOCK(KMYLOCK)
IF (OML_DEBUG) WRITE(0,'(1x,f20.6,1x,i3,a,2i20)') &
     & UTIL_WALLTIME(),OML_MY_THREAD(),': coml_SET_lockid <<',KMYLOCK,LOC_ADDR(KMYLOCK)
END SUBROUTINE COML_SET_LOCKID

SUBROUTINE COML_SET_LOCK()
USE OML_MOD, ONLY : OML_SET_LOCK
IMPLICIT NONE
CALL OML_SET_LOCK()
END SUBROUTINE COML_SET_LOCK

SUBROUTINE COML_UNSET_LOCKID(KMYLOCK)
USE OML_MOD, ONLY : OML_UNSET_LOCK, OML_LOCK_KIND, OML_MY_THREAD, OML_DEBUG
USE PARKIND1, ONLY : JPRD, JPIB
IMPLICIT NONE
INTEGER(KIND=OML_LOCK_KIND), INTENT(INOUT) :: KMYLOCK
INTEGER(KIND=JPIB), EXTERNAL :: LOC_ADDR
REAL(KIND=JPRD), EXTERNAL :: UTIL_WALLTIME
IF (OML_DEBUG) WRITE(0,'(1x,f20.6,1x,i3,a,2i20)') &
     & UTIL_WALLTIME(),OML_MY_THREAD(),': coml_UNSET_lockid >>',KMYLOCK,LOC_ADDR(KMYLOCK)
CALL OML_UNSET_LOCK(KMYLOCK)
IF (OML_DEBUG) WRITE(0,'(1x,f20.6,1x,i3,a,2i20)') &
     & UTIL_WALLTIME(),OML_MY_THREAD(),': coml_UNSET_lockid <<',KMYLOCK,LOC_ADDR(KMYLOCK)
END SUBROUTINE COML_UNSET_LOCKID

SUBROUTINE COML_UNSET_LOCK()
USE OML_MOD, ONLY : OML_UNSET_LOCK
IMPLICIT NONE
CALL OML_UNSET_LOCK()
END SUBROUTINE COML_UNSET_LOCK

SUBROUTINE COML_IN_PARALLEL(KISPAR_REGION)
USE PARKIND1, ONLY : JPIM
USE OML_MOD, ONLY : OML_IN_PARALLEL
IMPLICIT NONE
INTEGER(KIND=JPIM), INTENT(OUT) :: KISPAR_REGION
KISPAR_REGION = 0
IF (OML_IN_PARALLEL()) KISPAR_REGION = 1
END SUBROUTINE COML_IN_PARALLEL

SUBROUTINE COML_GET_MAX_THREADS(KTIDS)
USE PARKIND1, ONLY : JPIM
USE OML_MOD, ONLY : OML_MAX_THREADS
IMPLICIT NONE
INTEGER(KIND=JPIM), INTENT(OUT) :: KTIDS
KTIDS = OML_MAX_THREADS()
END SUBROUTINE COML_GET_MAX_THREADS

SUBROUTINE COML_GET_NUM_THREADS(KTIDS)
USE PARKIND1, ONLY : JPIM
USE OML_MOD, ONLY : OML_GET_NUM_THREADS
IMPLICIT NONE
INTEGER(KIND=JPIM), INTENT(OUT) :: KTIDS
KTIDS = OML_GET_NUM_THREADS()
END SUBROUTINE COML_GET_NUM_THREADS

SUBROUTINE COML_MY_THREAD(KMYTID)
USE PARKIND1, ONLY : JPIM
USE OML_MOD, ONLY : OML_MY_THREAD
IMPLICIT NONE
INTEGER(KIND=JPIM), INTENT(OUT) :: KMYTID
KMYTID = OML_MY_THREAD()
END SUBROUTINE COML_MY_THREAD
