! (C) Copyright 2014- ECMWF.
!
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
!
! In applying this licence, ECMWF does not waive the privileges and immunities
! granted to it by virtue of its status as an intergovernmental organisation
! nor does it submit to any jurisdiction.
MODULE EC_MPI_FINALIZE_MOD

CONTAINS

SUBROUTINE MEMINFO(KOUT,KSTEP)
USE PARKIND1, ONLY : JPIM, JPIB
USE EC_MEMINFO_MOD

IMPLICIT NONE
INTEGER(KIND=JPIM), INTENT(IN) :: KOUT, KSTEP
CHARACTER(LEN=32) CLSTEP
CHARACTER(LEN=160) :: LINE
CHARACTER(LEN=20) :: NODENAME
INTEGER(KIND=JPIB) :: NODE(0:17), ISMALL, IHUGE, ITOTAL
INTEGER(KIND=JPIM) :: I,INUMA,ICOMM
!#include "ec_meminfo.intfb.h"
WRITE(CLSTEP,'(11X,"STEP",I5," :")') KSTEP
ICOMM = -2 ! No headers from EC_MEMINFO by default
IF (KSTEP == 0) ICOMM = -1 ! Do print headers, too
CALL EC_MEMINFO(KOUT,TRIM(CLSTEP),ICOMM,KBARR=0,KIOTASK=-1,KCALL=-1)
CALL FLUSH(KOUT)
RETURN ! For now
#if 0
CALL EC_GETHOSTNAME(NODENAME) ! from support/env.c
OPEN(FILE="/proc/buddyinfo",UNIT=502,ERR=98,STATUS="old",ACTION="read")
READ(502,'(a)',END=99) LINE
READ(502,'(a)',END=99) LINE
DO INUMA=0,1
   NODE(:)=0
   READ(502,'(a)',END=99) LINE
   READ(LINE(22:160),*,ERR=99,END=99) NODE
   ISMALL = 0
   DO I=0,8
      ISMALL = ISMALL + NODE(I) * (2**I)
   ENDDO
   ! Pages >= 2M
   IHUGE = 0
   DO I=9,SIZE(NODE)-1
      IHUGE = IHUGE + NODE(I) * (2**I)
   ENDDO
   ITOTAL = ISMALL + IHUGE
   ISMALL = (ISMALL * 4096)/ONEMEGA
   IHUGE = (IHUGE * 4096)/ONEMEGA
   ITOTAL = (ITOTAL * 4096)/ONEMEGA
   WRITE(KOUT,'("   MEMINFO: STEP=",I0," ",A," NUMA# ",I0," : Free Total = SMALL + HUGEPAGES in MB: ",I0," = ",I0," + ",I0)') &
        & KSTEP, NODENAME, INUMA, ITOTAL, ISMALL, IHUGE
   WRITE(KOUT,'(" BUDDYINFO: STEP=",I0," ",A," NUMA# ",I0," : Count of free 2^(0..",I0,")*4096B blocks: ",A)') &
        & KSTEP, NODENAME, INUMA, SIZE(NODE)-1, LINE(22:160)
ENDDO
99 CONTINUE
CLOSE(502)
98 CONTINUE
CALL FLUSH(KOUT)
#endif
END SUBROUTINE MEMINFO

SUBROUTINE EC_MPI_FINALIZE(KERROR,LDCALLFINITO,LDMEMINFO,CALLER)
USE PARKIND1, ONLY : JPIM
USE MPL_MPIF
USE EC_MEMINFO_MOD

IMPLICIT NONE
INTEGER(KIND=JPIM), INTENT(OUT) :: KERROR
LOGICAL, INTENT(IN) :: LDCALLFINITO
LOGICAL, INTENT(IN) :: LDMEMINFO
CHARACTER(LEN=*), INTENT(IN) :: CALLER
LOGICAL :: LLINIT, LLFIN, LLNOTMPIWORLD
INTEGER(KIND=JPIM) :: IERR, ICOMM
INTEGER(KIND=JPIM) :: NCOMM_MEMINFO
COMMON /cmn_meminfo/ NCOMM_MEMINFO
!#include "ec_meminfo.intfb.h"
KERROR = 0
IF (LDCALLFINITO) THEN !*** common MPI_Finalize()
  CALL MPI_INITIALIZED(LLINIT,IERR)
  IF (LLINIT .AND. IERR == 0) THEN
    CALL MPI_FINALIZED(LLFIN,IERR)
    IF (.NOT.LLFIN .AND. IERR == 0) THEN
      LLNOTMPIWORLD = (NCOMM_MEMINFO /= 0 .and. NCOMM_MEMINFO /= MPI_COMM_WORLD)
      IF (LLNOTMPIWORLD) THEN
        ICOMM = NCOMM_MEMINFO
      ELSE
        ICOMM = MPI_COMM_WORLD
      ENDIF
      IF( LDMEMINFO ) CALL EC_MEMINFO(-1,"ec_mpi_finalize:"//caller,ICOMM,KBARR=1,KIOTASK=-1,KCALL=1)
      CALL c_drhook_prof() ! ifsaux/support/drhook.c : Make sure DrHook output is produced before MPI_Finalize (in case it fails)
      CALL MPI_BARRIER(ICOMM,IERR)
      IF (LLNOTMPIWORLD) THEN
        ! CALL MPI_COMM_FREE(NCOMM_MEMINFO,IERR)
        NCOMM_MEMINFO = 0
      ENDIF
      CALL MPI_FINALIZE(KERROR)
    ENDIF
  ENDIF
ENDIF
END SUBROUTINE EC_MPI_FINALIZE

SUBROUTINE EC_PMON(ENERGY,POWER)
USE PARKIND1, ONLY : JPIM, JPIB
IMPLICIT NONE
INTEGER(KIND=JPIB),INTENT(OUT) :: ENERGY,POWER
INTEGER(KIND=JPIB),SAVE :: ENERGY_START = 0
INTEGER(KIND=JPIM),SAVE :: MONINIT = 0
INTEGER(KIND=JPIM) :: ISTAT
CHARACTER(LEN=1) :: CLEC_PMON
ENERGY = 0
IF (MONINIT >= 0) THEN
   IF (MONINIT == 0) THEN ! The very first time only
      CALL GET_ENVIRONMENT_VARIABLE('EC_PMON',CLEC_PMON)
      IF (CLEC_PMON == '0') MONINIT = -2 ! Never try again
   ENDIF
   IF (MONINIT >= 0) THEN
      OPEN(503,FILE='/sys/cray/pm_counters/energy',IOSTAT=ISTAT,STATUS='old',ACTION='read')
      IF (ISTAT == 0) THEN
         READ(503,*,IOSTAT=ISTAT) ENERGY
         CLOSE(503)
         IF (ISTAT == 0) THEN
            IF (MONINIT == 0) THEN
               ENERGY_START = ENERGY
               MONINIT = 1 ! Ok
            ENDIF
            ENERGY = ENERGY - ENERGY_START
         ENDIF
      ENDIF
      IF (ISTAT /= 0) THEN
         MONINIT = -1 ! Never try again
         ENERGY = 0
      ENDIF
   ENDIF
ENDIF
POWER = 0
IF (MONINIT > 0) THEN
   OPEN(504,FILE='/sys/cray/pm_counters/power',IOSTAT=ISTAT,STATUS='old',ACTION='read')
   IF (ISTAT == 0) THEN
      READ(504,*,IOSTAT=ISTAT) POWER
      CLOSE(504)
   ENDIF
   IF (ISTAT /= 0) POWER = 0
ENDIF
END SUBROUTINE EC_PMON

END MODULE EC_MPI_FINALIZE_MOD
